import 'package:flutter/widgets.dart';
import 'package:flutter_riverpod/flutter_riverpod.dart';

import 'restorable_provider.dart';

/// A [ProviderScope] that automatically registers overriden RestorableProvider for restoration
class RestorableProviderScope extends ConsumerWidget {
  const RestorableProviderScope({
    required this.restorationId,
    this.overrides = const [],
    this.observers,
    this.parent,
    required this.child,
    Key? key,
  }) : super(key: key);

  /// {@macro flutter.widgets.widgetsApp.restorationScopeId}
  final String restorationId;
  final Widget child;

  final List<Override> overrides;
  final List<ProviderObserver>? observers;
  final ProviderContainer? parent;

  @override
  Widget build(BuildContext context, WidgetRef ref) {
    return ProviderScope(
      parent: parent,
      overrides: overrides,
      observers: observers,
      child: _RestorableProviderScope(
        restorationId: restorationId,
        overrides: overrides,
        child: child,
      ),
    );
  }
}

/// A [ProviderScope] that automatically registers overriden RestorableProvider for restoration
class _RestorableProviderScope extends ConsumerStatefulWidget {
  const _RestorableProviderScope({
    required this.restorationId,
    this.overrides = const [],
    required this.child,
    Key? key,
  }) : super(key: key);

  /// {@macro flutter.widgets.widgetsApp.restorationScopeId}
  final String restorationId;
  final Widget child;

  final List<Override> overrides;

  @override
  RestorableProviderScopeState createState() => RestorableProviderScopeState();
}

class RestorableProviderScopeState
    extends ConsumerState<_RestorableProviderScope> with RestorationMixin {
  @override
  Widget build(BuildContext context) => widget.child;

  @override
  String? get restorationId => widget.restorationId;

  @override
  void restoreState(RestorationBucket? oldBucket, bool initialRestore) {
    for (final override in widget.overrides) {
      if (override is RestorableOverride) {
        final value = ref.read(override.origin);
        if (value != null) {
          registerForRestoration(value, override.override.restorationId);
        }
      }
    }
  }
}
